@@ -11,7 +11,9 @@ use datafusion::datasource::TableProvider;
 use log::debug;
 use snafu::prelude::*;
 
-use crate::table::{self, TableLoadOption, TableOptionCsv, TableSource};
+use crate::table::{
+    self, datafusion_get_or_infer_schema, TableLoadOption, TableOptionCsv, TableSource,
+};
 
 #[derive(Debug, Snafu)]
 pub enum Error {
@@ -41,21 +43,30 @@ pub async fn to_datafusion_table(
         .option
         .clone()
         .unwrap_or_else(|| TableLoadOption::csv(TableOptionCsv::default()));
-    if opt.as_csv().unwrap().use_memory_table {
+    if opt
+        .as_csv()
+        .expect("Invalid table format option, expect csv")
+        .use_memory_table
+    {
         return to_mem_table(t, dfctx).await;
     }
     let table_url =
         ListingTableUrl::parse(t.get_uri_str()).with_context(|_| table::ListingTableUriSnafu {
             uri: t.get_uri_str().to_string(),
         })?;
-    let options = ListingOptions::new(Arc::new(CsvFormat::default()));
-    let schemaref = match &t.schema {
-        Some(s) => Arc::new(s.into()),
-        None => options
-            .infer_schema(&dfctx.state(), &table_url)
-            .await
-            .context(table::InferListingTableSchemaSnafu)?,
-    };
+    let mut options = ListingOptions::new(Arc::new(CsvFormat::default()));
+    if let Some(partition_cols) = t.datafusion_partition_cols() {
+        options = options.with_table_partition_cols(partition_cols)
+    }
+
+    let schemaref = datafusion_get_or_infer_schema(
+        dfctx,
+        &table_url,
+        &options,
+        &t.schema,
+        &t.schema_from_files,
+    )
+    .await?;
 
     let table_config = ListingTableConfig::new(table_url)
         .with_listing_options(options)
@@ -64,6 +75,7 @@ pub async fn to_datafusion_table(
         ListingTable::try_new(table_config).context(table::CreateListingTableSnafu)?,
     ))
 }
+
 pub async fn to_mem_table(
     t: &TableSource,
     dfctx: &datafusion::execution::context::SessionContext,
@@ -120,7 +132,7 @@ pub async fn to_mem_table(
         t,
         |r| -> Result<Vec<RecordBatch>, table::Error> {
             let mut builder = arrow::csv::reader::ReaderBuilder::new(schema_ref.clone())
-                .has_header(has_header)
+                .with_header(has_header)
                 .with_delimiter(delimiter)
                 .with_batch_size(batch_size);
             if let Some(p) = projection {
@@ -151,11 +163,12 @@ pub async fn to_mem_table(
 #[cfg(test)]
 mod tests {
     use super::*;
+    use datafusion::common::stats::Precision;
     use datafusion::prelude::SessionContext;
     use std::fs;
     use tempfile::Builder;
 
-    use crate::table::{TableIoSource, TableLoadOption};
+    use crate::table::TableIoSource;
     use crate::test_util::*;
 
     #[tokio::test]
@@ -189,8 +202,9 @@ mod tests {
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
-            .statistics();
-        assert_eq!(stats.num_rows, Some(37 * 3));
+            .statistics()
+            .unwrap();
+        assert_eq!(stats.num_rows, Precision::Exact(37 * 3));
     }
 
     #[tokio::test]
@@ -214,7 +228,8 @@ c1,c2,c3
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
-            .statistics();
-        assert_eq!(stats.num_rows, Some(3));
+            .statistics()
+            .unwrap();
+        assert_eq!(stats.num_rows, Precision::Exact(3));
     }
 }
