@@ -14,7 +14,9 @@ use datafusion::parquet::arrow::arrow_reader::ArrowReaderOptions;
 use datafusion::parquet::arrow::arrow_reader::ParquetRecordBatchReaderBuilder;
 use snafu::prelude::*;
 
-use crate::table::{self, TableLoadOption, TableOptionParquet, TableSource};
+use crate::table::{
+    self, datafusion_get_or_infer_schema, TableLoadOption, TableOptionParquet, TableSource,
+};
 
 #[derive(Debug, Snafu)]
 pub enum Error {
@@ -36,10 +38,6 @@ pub enum Error {
     EmptyPartition {},
     #[snafu(display("Schema not found"))]
     SchemaNotFound {},
-    #[snafu(display("Failed to infer table schema: {source}"))]
-    InferSchema {
-        source: datafusion::error::DataFusionError,
-    },
     #[snafu(display("Failed to parse table uri: {source}"))]
     ParseUri {
         source: datafusion::error::DataFusionError,
@@ -62,15 +60,19 @@ pub async fn to_datafusion_table(
         let table_url = ListingTableUrl::parse(t.get_uri_str())
             .context(ParseUriSnafu)
             .context(table::LoadParquetSnafu)?;
-        let options = ListingOptions::new(Arc::new(ParquetFormat::default()));
-        let schemaref = match &t.schema {
-            Some(s) => Arc::new(s.into()),
-            None => options
-                .infer_schema(&dfctx.state(), &table_url)
-                .await
-                .context(InferSchemaSnafu)
-                .context(table::LoadParquetSnafu)?,
-        };
+        let mut options = ListingOptions::new(Arc::new(ParquetFormat::default()));
+        if let Some(partition_cols) = t.datafusion_partition_cols() {
+            options = options.with_table_partition_cols(partition_cols)
+        }
+
+        let schemaref = datafusion_get_or_infer_schema(
+            dfctx,
+            &table_url,
+            &options,
+            &t.schema,
+            &t.schema_from_files,
+        )
+        .await?;
 
         let table_config = ListingTableConfig::new(table_url)
             .with_listing_options(options)
@@ -155,8 +157,8 @@ mod tests {
     use std::fs;
     use tempfile::Builder;
 
-    use crate::table::TableLoadOption;
     use crate::test_util::*;
+    use datafusion::common::stats::Precision;
     use datafusion::prelude::SessionContext;
 
     #[tokio::test]
@@ -179,12 +181,13 @@ mod tests {
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
-            .statistics();
-        assert_eq!(stats.num_rows, Some(500));
-        let stats = stats.column_statistics.unwrap();
-        assert_eq!(stats[0].null_count, Some(245));
-        assert_eq!(stats[1].null_count, Some(373));
-        assert_eq!(stats[2].null_count, Some(237));
+            .statistics()
+            .unwrap();
+        assert_eq!(stats.num_rows, Precision::Exact(500));
+        let stats = stats.column_statistics;
+        assert_eq!(stats[0].null_count, Precision::Exact(245));
+        assert_eq!(stats[1].null_count, Precision::Exact(373));
+        assert_eq!(stats[2].null_count, Precision::Exact(237));
 
         match t.as_any().downcast_ref::<ListingTable>() {
             Some(_) => {}
@@ -206,8 +209,9 @@ mod tests {
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
-            .statistics();
-        assert_eq!(stats.num_rows, Some(500));
+            .statistics()
+            .unwrap();
+        assert_eq!(stats.num_rows, Precision::Exact(500));
     }
 
     #[tokio::test]
@@ -236,7 +240,8 @@ mod tests {
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
-            .statistics();
-        assert_eq!(stats.num_rows, Some(1500));
+            .statistics()
+            .unwrap();
+        assert_eq!(stats.num_rows, Precision::Exact(1500));
     }
 }
