@@ -14,7 +14,8 @@ pub async fn post<H: RoapiContext>(
     body: Bytes,
 ) -> Result<impl IntoResponse, ApiErrResp> {
     let ctx = state.0;
-    let encode_type = encode_type_from_hdr(headers);
+    let response_format = H::get_response_format(&ctx).await;
+    let encode_type = encode_type_from_hdr(headers, response_format);
     let graphq = std::str::from_utf8(&body).map_err(ApiErrResp::read_query)?;
     let batches = ctx.query_graphql(graphq).await?;
     encode_record_batches(encode_type, &batches)
